﻿-- Garrison Visitor Radar
-- by Jadya
-- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local addon = VisitorRadar
local version = 1000

local addontitle = "|cff00AAFFVisitor Radar|r"
local vricon = "Interface\\icons\\INV_Sigil_Freya"

local pName = GetUnitName("player")
local pRealm = GetRealmName()
addon.pID = pName.."-"..pRealm

function addon:GetCharacterData(character)
 local result = VisitorRadar_CharData[character]

 if result then
  local vid = result.visitor and result.visitor.id
  local quests = result.visitor and result.visitor.quests
  local tamer = result.pet_tamer
  return result.t, vid, quests, tamer
 end
end

function addon:SetCharacterData(character, t, vid)
 if not VisitorRadar_CharData[character] then VisitorRadar_CharData[character] = {} end

 local result = VisitorRadar_CharData[character]
 if not result.visitor then result.visitor = {} end
 if not result.visitor.quests then result.visitor.quests = {} end
 result.t = t
 result.visitor.id = vid
 return result
end

-- npc detection
local faction = UnitFactionGroup("player")
local pet_tamer_ids = faction == "Alliance" and {91014,91015,91016,91017,90675} or {91361,91362,91363,91364,91026}
 
local updateFrame = CreateFrame("Frame")
local pet_tamer_found, visitor_found
local lastupdate = 0
local function update(self, elapsed)
 local zone = GetZoneText()
 if C_Garrison.IsUsingPartyGarrison() or (zone ~= GetMapNameByID(971) and zone ~= GetMapNameByID(976) and 
    zone ~= GARRISON_TOWN_HALL_HORDE and zone ~= GARRISON_TOWN_HALL_ALLIANCE) then
  return
 end

 
 local t = GetTime()
 if lastupdate < t - 0.1 then lastupdate = t else return end
 
  local function checkNPC(m, k)
   local guid = UnitGUID(m)
   
   if not guid then return end
   
   local _,_,_,_,_,_,_,id = string.find(guid, "(%a+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)")
   id = tonumber(id)
   
   return id == k
  end

 if not pet_tamer_found then
 for _,k in pairs(pet_tamer_ids) do
  if checkNPC("mouseover", k) or checkNPC("target", k) then
   VisitorRadar_CharData[addon.pID].pet_tamer = true
   pet_tamer_found = k
  end
 end
 end
 
 if not visitor_found then
 for k in pairs(addon.NPCs) do
  if checkNPC("mouseover", k) or checkNPC("target", k) then
   addon:SetCharacterData(addon.pID, time() + GetQuestResetTime(), k)
   addon:Pulse()
   visitor_found = k
  end
 end
 end
 
 if pet_tamer_found and visitor_found then
  updateFrame:Hide()
 end
end
updateFrame:SetScript("OnUpdate", update)
updateFrame:Show()

addon.eventframe = CreateFrame("FRAME","GVREventFrame")

local function addquest(t, q)
 for _,v in pairs(t) do
  if v == q then return end
 end
 table.insert(t, q)
end

addon.eventframe:RegisterEvent("QUEST_GREETING")
function addon.eventframe.QUEST_GREETING(...)

 if not visitor_found then return end
 
 local guid = UnitGUID("target")
   
 if not guid then return end
   
 local _,_,_,_,_,_,_,id = string.find(guid, "(%a+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)")
 id = tonumber(id)
 
 if not VisitorRadar_CharData[addon.pID].visitor.quests then
  VisitorRadar_CharData[addon.pID].visitor.quests = {}
 end
 if id == visitor_found then
  for i = 1,GetNumAvailableQuests() do
   addquest(VisitorRadar_CharData[addon.pID].visitor.quests, GetAvailableTitle(i))
  end
  --addon.eventframe:UnregisterEvent("QUEST_GREETING")
  --addon.eventframe:UnregisterEvent("QUEST_DETAIL")
 end
end

addon.eventframe:RegisterEvent("QUEST_DETAIL")
function addon.eventframe.QUEST_DETAIL()
 if not visitor_found then return end
 
 local guid = UnitGUID("target")
   
 if not guid then return end
   
 local _,_,_,_,_,_,_,id = string.find(guid, "(%a+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)")
 id = tonumber(id)
 
 if not VisitorRadar_CharData[addon.pID].visitor.quests then
  VisitorRadar_CharData[addon.pID].visitor.quests = {}
 end
 if id == visitor_found then
  addquest(VisitorRadar_CharData[addon.pID].visitor.quests, GetTitleText())
  --addon.eventframe:UnregisterEvent("QUEST_GREETING")
  --addon.eventframe:UnregisterEvent("QUEST_DETAIL")
 end
end

addon.eventframe:RegisterEvent("VARIABLES_LOADED")
function addon.eventframe.VARIABLES_LOADED()
 if VisitorRadar_CharData == nil then VisitorRadar_CharData = {} end
 if VisitorRadar_Options == nil then VisitorRadar_Options = {} end
 local O = VisitorRadar_Options

 -- default options
 if O["show_in_guildframe"] == nil then O["show_in_guildframe"] = true end
 if O["noguild"] == nil then O["noguild"] = false end
 --

 O["Version"] = version
 
 addonTable:SetupConfigFrame()

 local outdated = {}

 local now = time()
 for k,v in pairs(VisitorRadar_CharData) do
  local t = v.t
  local vid = v.visitor and v.visitor.id
  if t and now > t then
   --print(addontitle.." |cffFF0000debug|r - ("..k..") Visitor "..(L.ID2Name(vid) or "").." removed (outdated)")
   table.insert(outdated, k)
  end
 end
 
 for _,v in pairs(outdated) do
  VisitorRadar_CharData[v] = nil
 end
end

addon.eventframe:SetScript("OnEvent", function(self, event, ...) addon.eventframe[event](...) end)